@@ -12,17 +12,24 @@
 # by Carsten F. Dormann, 04.11.2005, carsten.dormann@ufz.de
 # Re-implementation allowing list representation
 # Roger Bivand 28.11.2005
-#
+# Upgrade to sp classes February 2007
 
 autocov_dist <- function(z, xy, nbs=1, type="inverse", zero.policy=FALSE,
-   style="W", lonlat=FALSE) {
+   style="W", longlat=NULL) {
    if (type=="one") expo <- 0
    if (type=="inverse") expo <- 1
    if (type=="inverse.squared") expo <- 2
-   nb <- dnearneigh(xy, 0, nbs, lonlat=lonlat)
+   if (inherits(xy, "SpatialPoints")) {
+      if ((is.null(longlat) || !is.logical(longlat)) 
+	 && !is.na(is.projected(xy)) && !is.projected(xy)) {
+         longlat <- TRUE
+      } else longlat <- FALSE
+      xy <- coordinates(xy)
+   } else if (is.null(longlat) || !is.logical(longlat)) longlat <- FALSE
+   nb <- dnearneigh(xy, 0, nbs, longlat=longlat)
    if (any(card(nb) == 0)) warning(paste("With value", nbs,
       "some points have no neighbours"))
-   nbd <- nbdists(nb, xy, lonlat=lonlat)
+   nbd <- nbdists(nb, xy, longlat=longlat)
    if (expo == 0) lw <- nb2listw(nb, style=style, zero.policy=zero.policy)
    else {
       gl <- lapply(nbd, function(x) 1/(x^expo))
