@@ -1,4 +1,4 @@
-# Copyright (c) 2007 Markus Reder and Roger Bivand
+# Copyright (c) 2007-2008 Markus Reder and Roger Bivand
 
 lm.morantest.exact <- function(model, listw, zero.policy = FALSE, 
     alternative = "greater", spChk=NULL, resfun=weighted.residuals, 
@@ -6,7 +6,7 @@ lm.morantest.exact <- function(model, listw, zero.policy = FALSE,
 {
     if (!inherits(listw, "listw")) 
         stop(paste(deparse(substitute(listw)), "is not a listw object"))
-    if (class(model) != "lm") 
+    if (!inherits(model, "lm")) 
         stop(paste(deparse(substitute(model)), "not an lm object"))
     N <- length(listw$neighbours)
     u <- resfun(model)
@@ -52,7 +52,6 @@ lm.morantest.exact <- function(model, listw, zero.policy = FALSE,
         res <- exactMoran(I, gamma, alternative=alternative, type="Global")
     } else {
         if (dim(Omega)[1] != N) stop("Omega of different size than data")
-        Omega <- chol(Omega)
         res <- exactMoranAlt(I, M, U, Omega, N, alternative=alternative,
             type="Alternative")
     }
@@ -95,7 +94,7 @@ exactMoran <- function(I, gamma, alternative="greater", type="Global", np2=NULL)
     else if (alternative == "greater")
         p.v <- pnorm(sd.ex, lower.tail=FALSE)
     else p.v <- pnorm(sd.ex)
-    if (is.nan(p.v) || p.v < 0 || p.v > 1) 
+    if (!is.finite(p.v) || p.v < 0 || p.v > 1) 
 	warning("Out-of-range p-value: reconsider test arguments")
     statistic <- sd.ex
     attr(statistic, "names") <- "Exact standard deviate"
@@ -113,7 +112,8 @@ exactMoran <- function(I, gamma, alternative="greater", type="Global", np2=NULL)
 
 exactMoranAlt <- function(I, M, U, Omega, n, alternative="greater",
     type="Alternative") {
-    A <- Omega %*% M %*% (U- diag(I, n)) %*% M %*% t(Omega)
+    Omega <- chol(Omega)
+    A <- Omega %*% M %*% (U - diag(I, n)) %*% M %*% t(Omega)
     gamma <- sort(eigen(A)$values)
     obj <- exactMoran(I, gamma, alternative=alternative,
         type=type)
@@ -127,3 +127,61 @@ print.moranex <- function(x, ...) {
     invisible(x)
 }
 
+H1_moments <- function(M, U, Omega, n) {
+    B <- Omega %*% M %*%  t(Omega)
+    eigen <- eigen(B)
+    lambda <- eigen$values
+    P <- eigen$vectors
+    A <- Omega %*% M %*% U %*% M %*% t(Omega)
+    H <- t(P) %*% A %*% P
+    hh <- diag(H)
+    integrand <- function(x) apply((1+2*lambda %*% t(x))^(-0.5),2,prod) *
+        colSums(hh/(1+2*lambda %*% t(x)))
+    mu <- integrate(integrand,lower=0, upper=Inf)$value
+    integrand2 <- function(x) {
+	res=0
+	for (i in 1:n){
+	    for(j in 1:n){
+		res=res+(H[i,i]*H[j,j]+2*H[i,j]^2) /
+                    ((1+2*lambda[i]*x)*(1+2*lambda[j]*x))*x
+	    }
+	}
+	apply((1+2*lambda %*% t(x))^(-0.5),2,prod)*res
+    }
+    mu2 <- integrate(integrand2,lower=0, upper=Inf)$value
+    res<-list(Ew=mu,Var=mu2-mu^2)
+    res
+}
+
+moranExpect_H1 <- function(listw, rho, select=FALSE){
+	if (!(select[1]))
+		select=1:length(listw$neighbours)
+	n <- length(select)
+	V <- listw2mat(listw)[select,select]
+	M <- diag(n)-matrix(rep(1/n,n*n),nrow=n)
+	WOm=invIrW(listw, rho=rho)[select,select]
+	B <- WOm %*% M %*%  t(WOm)
+	eigen <- eigen(B)
+	lambda <- eigen$values
+	P <- eigen$vectors
+	A <- WOm %*% M %*% (0.5*(V + t(V))) %*% M %*% t(WOm)
+	H <- t(P) %*% A %*% P
+	hh <- diag(H)
+	integrand <- function(x) apply((1+2*lambda %*% t(x))^(-0.5), 2, prod) *
+		colSums(hh/(1+2*lambda %*% t(x)))
+	mu <- integrate(integrand, lower=0, upper=Inf)$value
+	mu
+}
+
+moranExpect_rho_H1 <- function(listw, I0, select=FALSE){
+	if (!(select[1]))
+		select=1:length(listw$neighbours)
+	V <- listw2mat(listw)[select,select]
+	eigenvalues<-as.real(eigen(V)$values)
+	min=min(eigenvalues)
+	max=max(eigenvalues)
+	f <- function(x) abs(I0-moranExpect_H1(listw, x, select=select))
+	rop <- optimise(f, c(1/min,1/max))$minimum
+	rop
+}
+
